require_relative '../lib/stackoverflow_api_fetcher'
# require_relative 'fixtures/test_json'
require 'open-uri'
require 'json'

describe TdBerlinTest::StackexchangeApiFetcher do
  before(:each) do
    $stackoverflow_api_fetcher = TdBerlinTest::StackexchangeApiFetcher.new 'fixtures/langs.yaml'
  end

  describe '.run' do
    it 'should exist' do
      expect($stackoverflow_api_fetcher).to respond_to(:run)
    end
  end

  describe '.run' do
    it 'give the correct result for the test data' do
      $stackoverflow_api_fetcher.run
      expect($stackoverflow_api_fetcher.result['ruby']).to be > 100000
    end
  end

  describe '.parse_yaml_file' do
    it 'should parse the sample yaml file correctly' do
      expect($stackoverflow_api_fetcher).to respond_to(:parse_yaml_file)
      expect($stackoverflow_api_fetcher.language_data[0].keys[0]).to eq 'langs'
    end
  end

  describe '.fetch_stackoverflow' do
    it 'should return a xml response for ruby as input language' do
      xml_response = JSON.parse($stackoverflow_api_fetcher.fetch_stackoverflow('ruby'))
      expect(xml_response.keys.include?('items')).to be_truthy
    end
  end

  # TODO: move the content of test_json out into fixture
  describe '.get_tag_usage_count_from_json' do
    it 'should return a number greater than 100000' do
      test_json = '{
        "items":[
          {"has_synonyms":true,"is_moderator_only":false,"is_required":false,"count":190401,"name":"ruby-on-rails"},
          {"has_synonyms":false,"is_moderator_only":false,"is_required":false,"count":123770,"name":"ruby"}
        ],
        "has_more":true,
        "quota_max":300,
        "quota_remaining":225
      }'

      ruby_tag_count = $stackoverflow_api_fetcher.get_tag_usage_count_from_json(test_json, 'ruby')
      expect(ruby_tag_count).to be > 100000
    end
  end

  describe '.sort_result_by_tag_usage_count' do
    it 'should sort the items of an input hash by value of items' do
      result_hash = $stackoverflow_api_fetcher.sort_result_by_tag_usage_count({'a' => 3, 'b' => 5})
      expect(result_hash).to eq({'b' => 5, 'a' => 3})
    end
  end
end